@@ -10,7 +10,7 @@ is.even <- function(x) {
 grid.pretty <- function(range) {
   if (!is.numeric(range))
     stop("range must be numeric")
-  .Call("L_pretty", range)
+  .Call("L_pretty", range, PACKAGE="grid")
 }
 
 
