@@ -1,7 +1,7 @@
 
 .First.lib <- function(lib, pkg) {
   library.dynam( "grid", pkg, lib )
-  .Call("L_initGrid")
+  .Call("L_initGrid", PACKAGE="grid")
   .grid.loaded <<- TRUE
 }
 
@@ -12,7 +12,10 @@
     # Not very friendly to other registered graphics systems
     # but its safety first for now
     graphics.off()
-    .Call("L_killGrid")
+    .Call("L_killGrid", PACKAGE="grid")
+    dyn.unload(paste(libpath, "libs",
+                     paste("grid", .Platform$dynlib.ext, sep=""),
+                     sep=.Platform$file.sep))
   }
 }
 
